/*
*
* Gtk+1.2 Utility routines
* Copyright (C) 2002-2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/
#ifndef	__HAVE_MYGTK_
#define	__HAVE_MYGTK_    1

#include <gtk/gtk.h>
#include "mylib.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


/********************* Debug utility *****************************************/

/* This is ugly but saves a lot of lines and ifdefs */

#ifdef DEBUG
	#define debug_puts		puts
	#define debug_printf		printf
#else
int debug_puts(char *msg){return EXIT_SUCCESS;}
int debug_printf(const char * format, ... ){return EXIT_SUCCESS;}
#endif

/****************End of Debug utility ****************************************/

/********************* Text box widget ***************************************/

#define EDITABLE					1
#define NOT_EDITABLE				0

GtkWidget *text_box_create_on_table(char *text, int editable, GtkWidget *table,
								 int start_col, int end_col, int start_row, int end_row)
{
	GtkWidget *entry_box;
	entry_box = gtk_entry_new();
	gtk_table_attach_defaults (GTK_TABLE(table),entry_box, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	if(text!=NULL)
		gtk_entry_set_text(GTK_ENTRY(entry_box),text);
	gtk_entry_set_editable (GTK_ENTRY (entry_box),(editable==EDITABLE)?TRUE:FALSE);
	return entry_box;
}

/********************* End of Text box widget ********************************/

/********************* Standard radio button widget **************************/

#define ACTIVE 		TRUE
#define INACTIVE		FALSE

GtkWidget *radio_create(GtkWidget *precedent_of_group, char *label,
			 GtkSignalFunc func, gpointer func_data, int active )
{
	GtkWidget *button;

	if(precedent_of_group==NULL)
		button = gtk_radio_button_new_with_label(NULL, label);
	else
		button = gtk_radio_button_new_with_label(gtk_radio_button_group (GTK_RADIO_BUTTON (precedent_of_group)), label);
 	if(func==NULL)
	{
		fprintf(stderr,"radio_create(): GtkSignalFunc func == NULL\n");
		exit(EXIT_FAILURE);
	}
	gtk_signal_connect (GTK_OBJECT (button), "toggled",
								GTK_SIGNAL_FUNC(func), func_data);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),(active==ACTIVE)?TRUE:FALSE);
	return button;
}

/****************End of Standard radio button widget *************************/

/********************* Radio button on table widget **************************/

GtkWidget *radio_create_on_table(GtkWidget *precedent_of_group, char *label, GtkSignalFunc func, gpointer func_data,
										int active, GtkWidget *table, int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *button;
	button = radio_create(precedent_of_group, label, func, func_data ,active );
	/* We use abs to handle illegal negative values. */
	gtk_table_attach_defaults (GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return button;
}

GtkWidget *default_radio_create_on_table(GtkWidget *precedent_of_group, char *label, GtkSignalFunc func,/* gpointer func_data,*/
										int active, GtkWidget *table, int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *button;
	button = radio_create(precedent_of_group, label, func, /*func_data*/label ,active );
	/* We use abs to handle illegal negative values. */
	gtk_table_attach_defaults (GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return button;
}

/****************End of radio button on table widget *************************/

/********************* Standard check button widget **************************/

GtkWidget *check_create(char *label, GtkSignalFunc func, gpointer func_data, int active )
{
	GtkWidget *button;

	button = gtk_check_button_new_with_label(label);
	if(func==NULL)
	{
		fprintf(stderr,"check_create(): GtkSignalFunc func == NULL\n");
		exit(EXIT_FAILURE);
	}
	gtk_signal_connect (GTK_OBJECT (button), "toggled",
								GTK_SIGNAL_FUNC(func), func_data);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),(active==ACTIVE)?TRUE:FALSE);
	return button;
}

/****************End of Standard check button widget *************************/

/********************* Check button on table widget ***************************/

GtkWidget *check_create_on_table(char *label, GtkSignalFunc func, gpointer func_data, int active,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *button;
	button = check_create(label, func, func_data ,active );
	/* We use abs to handle illegal negative values. */
	gtk_table_attach_defaults (GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return button;
}

GtkWidget *default_check_create_on_table(char *label, GtkSignalFunc func, int active,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *button;
	button = check_create(label, func, label ,active );
	/* We use abs to handle illegal negative values. */
	gtk_table_attach_defaults (GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return button;
}
/***************End of Check button on table widget **************************/

/********************* Standard button widget ********************************/

GtkWidget *button_create(char *label, GtkSignalFunc func, gpointer func_data )
{
	GtkWidget *button;

	button = gtk_button_new_with_label (label);
	if(func==NULL)
	{
		fprintf(stderr,"button_create(): GtkSignalFunc func == NULL\n");
		exit(EXIT_FAILURE);
	}
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
								GTK_SIGNAL_FUNC (func), func_data);
	return button;
}

/****************End of Standard button widget *******************************/

/********************* Button on table widget ********************************/

GtkWidget *button_create_on_table(char *label, GtkSignalFunc func, gpointer func_data,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *button;

	button = button_create(label, func, func_data );
	/* We use abs to handle illegal negative values. */
	gtk_table_attach_defaults (GTK_TABLE(table), button, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return button;
}
/****************End of button on table widget ******************************/

/********************* Button in container widget ***************************/

GtkWidget *button_create_in_container(char *label, GtkSignalFunc func, gpointer func_data,GtkWidget *container)
{
	GtkWidget *button;

	button = button_create(label, func, func_data);
	gtk_container_add (GTK_CONTAINER (container), button);
	return button;
}

/*************End of Button in container widget *****************************/

/********************* Buttonbox on table widget ****************************/

#define HORIZONTAL				0
#define VERTICAL					1

GtkWidget *button_box_create_on_table(int orientation, int layout,  int border,
										 int spacing, int child_size_x, int child_size_y, GtkWidget *table,
										 int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *bbox;

	if(orientation==HORIZONTAL)
		bbox = gtk_hbutton_box_new ();
	else if(orientation==VERTICAL)
		bbox = gtk_vbutton_box_new ();
	else
	{
		fprintf(stderr,"button_box_create_on_table(): orientation != HORIZONTAL or VERTICAL \n");
		exit(EXIT_FAILURE);
	}
	gtk_container_set_border_width (GTK_CONTAINER (bbox),abs(border));
	gtk_table_attach_defaults (GTK_TABLE(table), bbox, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), layout );
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (bbox),abs(spacing));
	gtk_button_box_set_child_size (GTK_BUTTON_BOX (bbox),abs(child_size_x),abs(child_size_y));
	return bbox;
}

GtkWidget *default_button_box_create_on_table(int orientation, GtkWidget *table,
										 int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *bbox;

	/* These default values look good to me. If you don't like it
		use button_box_create_on_table()	to set your own */
	bbox = button_box_create_on_table( orientation,  GTK_BUTTONBOX_END, 2, 20, 85, 20,
								 					table, start_col, end_col, start_row, end_row );
	return bbox;
}

/***************End of Buttonbox on table widget ****************************/

/********************* Label on table widget ********************************/

GtkWidget *label_create_on_table(char *label_text,GtkWidget *table,
				int start_col, int end_col, int start_row, int end_row )
{
	GtkWidget *label;
	label=gtk_label_new(label_text);
	gtk_table_attach_defaults (GTK_TABLE(table), label, abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return label;
}
/***************End of Label on table widget ********************************/

/********************* Standard table widget ********************************/

GtkWidget *table_create(int rows, int cols , GtkWidget* window , int border)
{
	GtkWidget *table;

	if(rows <= 0) rows=1;
	if(cols <= 0) cols=1;

	if(window==NULL)
	{
		fprintf(stderr,"table_create(): GtkWidget *window == NULL\n");
		exit(EXIT_FAILURE);
	}
	table = gtk_table_new (rows, cols, TRUE);
	gtk_container_add (GTK_CONTAINER (window), table);
	gtk_container_set_border_width (GTK_CONTAINER (table),abs(border) );
	return table;
}

/********************* Standard table widget ********************************/

/********************* Standard window widget ********************************/

#define	NOT_MODAL						0
#define	MODAL								1

GtkWidget *window_create( int x, int y, int border, char *title, GtkSignalFunc func,  gpointer func_data, int modal)
{
		GtkWidget	*window;
		window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		if(func==NULL)
		{
			fprintf(stderr,"window_create(): GtkSignalFunc func == NULL\n");
			exit(EXIT_FAILURE);
		}
		gtk_signal_connect (GTK_OBJECT (window), "delete_event",
									GTK_SIGNAL_FUNC (func), func_data);
		if(x==0)x=640;
		if(y==0)y=480;

		gtk_widget_set_usize(GTK_WIDGET(window),abs(x),abs(y));
		gtk_window_set_title(GTK_WINDOW(window), title);
		gtk_container_set_border_width (GTK_CONTAINER (window), abs(border));
		gtk_window_set_modal(GTK_WINDOW(window),(modal==MODAL) ? TRUE : FALSE);
		return window;
}

/********************* Standard window widget ********************************/

/********************* For the file selection widget *************************/

#define	HIDE_FILE_OP_BUTTONS			0
#define	SHOW_FILE_OP_BUTTONS			1

char *selected_path=NULL;
GtkWidget	*file_sel_win=NULL;

void file_sel_ok(GtkWidget *widget,GtkFileSelection *file_selection);
char *file_selector(char *title, int fileop, char *ptr);

#include	"FileSelector.c"

/*************End of the file selection widget********************************/

/********************* Frame on table widget ********************************/

GtkWidget *frame_create_on_table(char *label, GtkWidget *table,
											 int start_col,int end_col, int start_row, int end_row )
{
	GtkWidget *frame;

	frame = gtk_frame_new(label);
	gtk_table_attach_defaults(GTK_TABLE(table), frame,  abs(start_col), abs(end_col), abs(start_row), abs(end_row));
	return frame;
}

/**************End of Frame on table widget ********************************/

/********************* Dialog window widget **********************************/

#define	ONE_BUTTON						1
#define	TWO_BUTTONS						2
#define	THREE_BUTTONS					3

int dialog_retval;

void dialog_callback(GtkWidget *widget, gpointer callback_data );
int dialog_window(char *title, char *labeltext , int type, int modal, char *third_button_label);

#include "DialogWindow.c"

/********************* Dialog window widget **********************************/

/********************* Text file viewer Widget *******************************/
#define WRAP				0
#define NO_WRAP			1
#define SCROLL				0
#define NO_SCROLL			1
/*
already defined for Text box widget
#define EDITABLE					1
#define NOT_EDITABLE				0
*/

GtkWidget *textpad_create_on_table(int editable, GtkSignalFunc func,  int wrap, int scroll, GtkWidget *table,
										  int start_col,int end_col, int start_row, int end_row )
{
	GtkWidget	*new_text,
					*vscrollbar,
					*hscrollbar;

	new_text = gtk_text_new(NULL, NULL);
	gtk_text_set_editable (GTK_TEXT (new_text), FALSE);
	if(editable==EDITABLE)
	{
		gtk_text_set_editable (GTK_TEXT (new_text), TRUE);
		/* we have to connect a signal here to have Gtk tell us if the text gets modified! */
		if(func==NULL)
		{
			fprintf(stderr,"text_create_on_table(): GtkSignalFunc func == NULL\n");
			exit(EXIT_FAILURE);
		}
		gtk_signal_connect (GTK_OBJECT (new_text),"changed", GTK_SIGNAL_FUNC(func),NULL);
	}

	gtk_text_set_word_wrap(GTK_TEXT(new_text),(wrap==WRAP)?TRUE:FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), new_text, abs(start_col), abs(end_col),
																			abs(start_row),abs(end_row));
	if(scroll==SCROLL)
	{
		vscrollbar = gtk_vscrollbar_new(GTK_TEXT(new_text)->vadj);
		hscrollbar = gtk_hscrollbar_new(GTK_TEXT(new_text)->hadj);
		gtk_table_attach(GTK_TABLE(table), vscrollbar,	abs(end_col), abs(end_col)+1,
																		abs(start_row),abs(end_row),
																		GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
		gtk_table_attach(GTK_TABLE(table), hscrollbar,	abs(start_col), abs(end_col),
																		abs(end_row),abs(end_row)+1,
																		GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
	}
	gtk_text_freeze(GTK_TEXT(new_text));
	gtk_widget_realize(new_text);
	gtk_text_thaw(GTK_TEXT(new_text));
	return new_text;
}

/* We need a wrapper for TEXT_CLEAR(x) to work 'cause I've not found a way to pass the GTK_TEXT(x) arg at runtime */
#define TEXT_CLEAR(x) gtk_text_backward_delete(GTK_TEXT (x), gtk_text_get_length(GTK_TEXT (x)))


void textview_window_create(char * title,int x, int y, int border, int modal_flag, FILE *fd, char *ptr);

#include "TextViewWindow.c"

/**************End of Text file viewer Widget ********************************/

#endif



