/*
 * Various utility routines put togheter by Tito <tito-wolit@tiscali.it> .
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef	__HAVE_MYLIB_
#define	__HAVE_MYLIB_    1

/*
 * Utility routines.
 *
 * Copyright (C) 1999,2000,2001 by Erik Andersen <andersee@debian.org>
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

extern void *xmalloc(size_t size)
{
	void *ptr = malloc(size);
	if (ptr == NULL && size != 0)
	{
		fprintf(stderr,"malloc(): memory exhausted");
		exit(EXIT_FAILURE);
	}
	return ptr;
}

extern void *xrealloc(void *ptr, size_t size)
{
	ptr = realloc(ptr, size);
	if (ptr == NULL && size != 0)
	{
		fprintf(stderr,"realloc(): memory exhausted\n");
		exit(EXIT_FAILURE);
	}
	return ptr;
}

extern void *xcalloc(size_t nmemb, size_t size)
{
	void *ptr = calloc(nmemb, size);
	if (ptr == NULL && nmemb != 0 && size != 0)
		fprintf(stderr,"calloc(): memory exhausted\n");
	return ptr;
}

FILE *xfopen(const char *path, const char *mode)
{
	FILE *fp;
	if ((fp = fopen(path, mode)) == NULL)
	{
		fprintf(stderr,"xfopen(): failed on: %s", path);
		exit(EXIT_FAILURE);
	}
	return fp;
}

FILE *xpopen(const char *path, const char *mode)
{
	FILE *fp;
	if ((fp = popen(path, mode)) == NULL)
	{
		fprintf(stderr,"xpopen(): failed on: %s", path);
		exit(EXIT_FAILURE);
	}
	return fp;
}

/*
 * xgetcwd.c -- return current directory with unlimited length
 * Copyright (C) 1992, 1996 Free Software Foundation, Inc.
 * Written by David MacKenzie <djm@gnu.ai.mit.edu>.
 *
 * Special function for busybox written by Vladimir Oleynik <dzo@simtreas.ru>
 * Hacked by Tito <tito-wolit@tiscali.it> for xfprot.
*/

#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <limits.h>
#include <sys/param.h>

/* Amount to increase buffer size by in each try. */
#define PATH_INCR 32

/* Return the current directory, newly allocated, arbitrarily long.
   Return NULL and set errno on error.
   If argument is not NULL (previous usage allocate memory), call free()
*/

char * xgetcwd (char *cwd)
{
  char *ret;
  unsigned path_max;

  path_max = (unsigned) PATH_MAX;
  path_max += 2;                /* The getcwd docs say to do this. */

  if(cwd==0)
	cwd = xmalloc (path_max);

  while ((ret = getcwd (cwd, path_max)) == NULL && errno == ERANGE) {
      path_max += PATH_INCR;
      cwd = xrealloc (cwd, path_max);
  }

  if (ret == NULL) {
      free (cwd);
      perror("getcwd()");
      return NULL;
  }

  return cwd;
}

/*
* Copyright (C) 2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This function checks strings for illegal chars that could
* be misinterpetrated if passed to a shell by a system() call.
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

char *illegal_chars_in_string(char *string,int substitute)
{
	int illegal_chars=0;
	int i=0;

	if(string==NULL)
		return NULL;

	/* Don't allow leading dash */
	if(string[0]== '-')
	{
		if(substitute==1)
			string[0]='_';
		illegal_chars++;
	}

	for(i=0 ; i<=strlen(string) ; i++)
	{
		switch(string[i])
		{
			case ' '	:
			case '/'	:
			case 92	: /*case '\':*/
			case '*'	:
			case '?'	:
			case ':'	:
			case ';'	:
			case ','	:
			case 39	: /*case ''': */
			case '`'	:
			case '<'	:
			case '>'	:
			case '"'	:
			case '!'	:
			case '$'	:
			case '%'	:
			case '&'	:
			case '='	:
			case '~'	:
			case '^'	:
			case '|'	:
			case '@'	:
			case '#'	:
			case '('	:
			case ')'	:
			case '['	:
			case ']'	:
			case '{'	:
			case '}'	:
				illegal_chars++;
				if(substitute==1)
					string[i]='_';
				break;
			case '-'	:
				/* Don't allow double dash */
			case '.'	:
				/* Don't allow double dot */
				if(string[i]==string[i+1])
				{
					illegal_chars++;
					if(substitute==1)
					{
						string[i]='_';
						string[i+1]='_';
					}
				}
				break;
			/*esac*/
		}/*switch*/
		i++;
	}/*for*/
	if(illegal_chars>0 && substitute==0)
		return NULL;
	return string;
}

/*
* Copyright (C) 2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
*  This function reads chars from a fd until EOF storing them into
*  a properly sized buffer and returns a pointer to this buffer .
*  The buffer should be freed .
*/

char * fgetc_buf(FILE *fd)
{
	int c, i=0;
	int size=1024;
	char *buffer=NULL;

	buffer=xmalloc(size);
	while((c=fgetc(fd))!=EOF)
	{
		buffer[i]=c;
		if(i==(size - 2))
			buffer=xrealloc(buffer,size+=1024);
		i++;
	}
	buffer[i]='\0';
	return buffer;
}

/*
* Copyright (C) 2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This function changes the current directory to that specified in path
* and exits on failure or returns -1 accordingly to int flag.
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

int xchdir(char *path, int flag)
{
	if((chdir(path))==0)
		return EXIT_SUCCESS;
	fprintf(stderr,"Cannot chdir() to %s: %s\n",path,strerror(errno));
	if(flag==EXIT_ON_FAILURE)
		exit(EXIT_FAILURE);
	return EXIT_ERROR;
}

/*
* Copyright (C) 2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This function  attempts to create a directory named pathname.
* mode  specifies the permissions to use. It is modified by the process's
* umask in the usual way and exits on failure or returns -1 accordingly to int flag.
*/

#include <unistd.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>

int xmkdir(const char *pathname , mode_t mode, int flag)
{
	if((mkdir(pathname, mode))==0)
		return EXIT_SUCCESS;
	fprintf(stderr,"Cannot mkdir(): %s with mode %#o: %s\n",pathname,mode,strerror(errno));
	if(flag==EXIT_ON_FAILURE)
		exit(EXIT_FAILURE);
	return EXIT_ERROR;
}

/*
* Copyright (C) 2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* The  owner of the file specified by path is changed.This function
* exits on failure or returns -1 accordingly to int flag.
*/

#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>

int xchown(const char *path, uid_t owner, gid_t group, int flag)
{
	if((chown(path, owner, group ))==0)
		return EXIT_SUCCESS;
	fprintf(stderr,"Cannot chown(): %s to uid %d and gid %d: %s\n",path,owner,group,strerror(errno));
	if(flag==EXIT_ON_FAILURE)
		exit(EXIT_FAILURE);
	return EXIT_ERROR;
}

/*
* Copyright (C) 2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
*  The  mode of the file given by path is changed.This function
*  exits on failure or returns -1 accordingly to int flag.
*/

#include <unistd.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>

int xchmod(const char *path, mode_t mode, int flag)
{
	if((chmod(path, mode ))==0)
		return EXIT_SUCCESS;
	fprintf(stderr,"Cannot chmod(): %s : %s\n",path,strerror(errno));
	if(flag==EXIT_ON_FAILURE)
		exit(EXIT_FAILURE);
	return EXIT_ERROR;
}

#endif
