/*
*
* XFPROT Copyright (C) 2002-2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* XFPROT is a graphical frontend for the F-Prot Antivirus
* for Linux Small Business Edition. It was based upon the
* XSTEP-3.5.1 library until version 0.19beta.
* Starting from version 0.20beta it is based upon Gtk-1.2.10.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/
#include "config.h"

#include <gtk/gtk.h>
#include "mylib.h"
#include "mygtk.h"
#include <fcntl.h>
#include <pwd.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

#include "xfprot.h"

#ifdef NO_CORE_DUMPS
	#include <sys/prctl.h>
#endif

#define MAX_PATH										PATH_MAX+2
#define MAX_NAME										NAME_MAX+2

#define SCAN_FOR_MACRO_VIRUS  					"Yes"
#define NOT_SCAN_FOR_MACRO_VIRUS  				"No"
#define ONLY_SCAN_FOR_MACRO_VIRUS	  			"Only"
#define USE_HEURISTICS								"Yes"
#define NOT_USE_HEURISTICS							"No"
#define ONLY_USE_HEURISTICS						"Only"
#define DEF_SIZE										64

int	read_license=0;

char	*command,
		*path,
		*report_file,
		flag_scanmode[DEF_SIZE],
		flag_disinf[DEF_SIZE],
		flag_rename[DEF_SIZE],
		flag_delete[DEF_SIZE],
		flag_file_types[DEF_SIZE],
		flag_archives[DEF_SIZE],
		flag_comp_exe[DEF_SIZE],
		flag_nosub[DEF_SIZE],
		flag_collect[DEF_SIZE],
		flag_list[DEF_SIZE],
		flag_ai[DEF_SIZE],
		flag_old[DEF_SIZE],
		flag_follow[DEF_SIZE],
		flag_append[DEF_SIZE],
		flag_heur[DEF_SIZE],
		flag_macro[DEF_SIZE],
		flag_macro_action[DEF_SIZE];

FILE	*fd;

struct	passwd *pw;
struct	stat stat_p;

GtkWidget		*path_entry_box,
					*report_path_entry_box,
					*action_table,
					*disinfect_button,
					*rename_button,
					*delete_button,
					*macro_remove_table,
					*macro_1_button,
					*macro_2_button,
					*macro_3_button;

/*#################################### File Selection Widget wrapper  ##############################################*/

void do_file_selection()
{
	debug_puts("do_file_selection()");

	path=file_selector("Select path to scan:", HIDE_FILE_OP_BUTTONS, path);

	debug_printf("do_file_selection(): (path %s= NULL): %s\n",(path)? "!" : "=",(path)? path : "NULL" );

	if(path!=NULL)
	{
		gtk_entry_set_text(GTK_ENTRY(path_entry_box),path);
		free(path);
		path=NULL;
	}
	/*else  path_entry_box unchanged */
}

/*############################### End of File Selection Widget wrapper #############################################*/


/*############################### Fprot-info, Virus-list, Update, Test & Reprot button functions ###################*/

void run(char *command_name)
{
	debug_printf("run() %s\n",command_name);
	system(command_name);
}

void fprot_info()
{
	char *tmp=NULL;

	debug_puts("fprot_info()");

	fd=popen(PATH_TO_F_PROT_BIN" "INFO_CMD,"r");

	/* Something bad happened */
	if(fd==NULL)
		return;

	tmp=fgetc_buf(fd);
	pclose(fd);
	debug_puts(tmp);
	if(strstr(tmp,"F-PROT ANTIVIRUS")!=NULL)
	{
		textview_window_create("Info",530,430,8,MODAL, NULL, tmp);
	}
	else
	{
		/* This is for older versions of f-prot which don't have -verno */
		debug_puts("Error: -verno is an unknown command-line switch, using -virno");
		fd=popen(PATH_TO_F_PROT_BIN" "INFO_CMD2,"r");
		textview_window_create("Info",530,430,8,MODAL,fd, NULL);
		if(fd)
			pclose(fd);
	}
	free(tmp);
}

void fprot_virlist()
{
	debug_puts("fprot_virlist()");

	fd=popen(PATH_TO_F_PROT_BIN" "VIRLIST_CMD,"r");
	textview_window_create("Virus List",530,430,8,MODAL,fd ,NULL);
	if(fd)
		pclose(fd);
}

void fprot_update()
{
	debug_puts("fprot_update()");

	if(lstat(UPDATE_CMD_VER_4_1,&stat_p)==0)
	{
		/* For f-prot Program version: 4.1.0 */
		run(TERMINAL" "UPDATE_CMD_VER_4_1" &");
	}
	else
	{
		/* For f-prot Program version:< 4.1.0 */
		run(TERMINAL" "UPDATE_CMD" &");
	}
}

void fprot_test()
{
	debug_puts("fprot_test()");

	fd=popen(PATH_TO_F_PROT_BIN" "VIRTEST_CMD,"r");
	textview_window_create("Test",530,430,8,MODAL,fd ,NULL);
	if(fd)
		pclose(fd);
}

void prepare_report_path()
{
	debug_puts("prepare_report_path()");

	/* Prepare report file name and check for
	 illegal  chars and substitute them with '_' */
	gtk_entry_set_text(GTK_ENTRY(report_path_entry_box),
			illegal_chars_in_string(gtk_entry_get_text
			(GTK_ENTRY(report_path_entry_box)),SUBSTITUTE));
	/* Prepare report path */
	report_file=(char *)xmalloc(MAX_PATH);
	report_file=xgetcwd(report_file);
	report_file=xrealloc(report_file,strlen(report_file)+strlen("/")+
				strlen(gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)))+1);
	strcat(report_file,"/");
	strcat(report_file,gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)));

	debug_printf("report_file: %s\n",report_file);

}

void view_report()
{
	debug_puts("view_report()");

	prepare_report_path();
	fd = fopen(report_file, "r");
	textview_window_create(report_file,530,430,8,MODAL,fd,NULL);
	if(fd)
		fclose(fd);
	free(report_file);
}

/*######################### END of Fprot-info, Virus-list, Update, Test & Report button functions ###################*/

/*############################# START of Scan button function  ######################################################*/

void add_to_command(char *option)
{
	int size;
	size=strlen(option)+strlen(command)+1;
	command=(char*)xrealloc(command,size);
	strncat(command,option,size);
	debug_printf("add_to_command() %s\n",command);
}

void do_scan_f()
{
	debug_puts("do_scan_f()");
	/* Don't change to "f-prot","/usr/local/f-prot/f-prot.sh" or "/usr/bin/f-prot"
		this will break things */

	command=(char *)xmalloc(strlen(TERMINAL)+strlen(PATH_TO_F_PROT_BIN)+
				strlen(gtk_entry_get_text(GTK_ENTRY(path_entry_box)))+1+2+1+1);

	/* We always "" quote the path to scan to avoid troubles with filenames with spaces
		which are split by the shell and mess up our command line args */

	sprintf(command,"%s %s \"%s\"",TERMINAL,PATH_TO_F_PROT_BIN,
						gtk_entry_get_text(GTK_ENTRY(path_entry_box)));

	/* I prefer to create always a report file so no flag checking here.*/
	/* BTW we always quote filenames and paths */
	add_to_command(" -report=");
	add_to_command("\"");
	add_to_command(report_file);
	add_to_command("\"");

	/* Let's parse the options*/
	if(strcmp(flag_scanmode,"Report only")!=0)
	{
		if(strcmp(flag_disinf,"Disinfect")==0)
			add_to_command(" -disinf");
		if(strcmp(flag_rename,"Rename")==0)
			add_to_command(" -rename");
		if(strcmp(flag_delete,"Delete")==0)
			add_to_command(" -delete");
		if(strcmp(flag_scanmode,"Automatic")==0)
			add_to_command(" -auto");
	}
	if(strcmp(flag_file_types,"Attempt to identify files")==0)
		add_to_command(" -type");
	if(strcmp(flag_file_types,"Standard file extensions")==0)
		add_to_command(" -ext");
	if(strcmp(flag_file_types,"Scan all files")==0)
		add_to_command(" -dumb");
	if(strcmp(flag_archives,"Scan inside archives")==0)
		add_to_command(" -archive");
	if(strcmp(flag_comp_exe,"Scan compressed executables")==0)
		add_to_command(" -packed");
	if(strcmp(flag_nosub,"Do not scan subdirectories")==0)
		add_to_command(" -nosub");
	if(strcmp(flag_collect,"Scan for other various malware")==0)
		add_to_command(" -collect");
	if(strcmp(flag_list,"List all scanned files")==0)
		add_to_command(" -list");
	if(strcmp(flag_ai,"Enable neural network virus detection")==0)
		add_to_command(" -ai");
	if(strcmp(flag_old,"Do not complain about old DEF files")==0)
		add_to_command(" -old");
	/* "Follow symbolic links" is not displayed in Switches: <none>
		in f-prot Program version: 4.1.1 Engine version: 3.13.3
		but works correctly	*/
	if(strcmp(flag_follow,"Follow symbolic links")==0)
		add_to_command(" -follow");
	if(strcmp(flag_append,"Append to existing report file")==0)
		add_to_command(" -append");
	if(strcmp(flag_heur,NOT_USE_HEURISTICS)==0)
		add_to_command(" -noheur");
	if(strcmp(flag_heur,ONLY_USE_HEURISTICS)==0)
		add_to_command(" -onlyheur");
	if(strcmp(flag_macro,NOT_SCAN_FOR_MACRO_VIRUS)==0)
		add_to_command(" -nomacro");
	if(strcmp(flag_macro,ONLY_SCAN_FOR_MACRO_VIRUS)==0)
		add_to_command(" -onlymacro");
	if(strcmp(flag_scanmode,"Report only")!=0 && strcmp(flag_macro,NOT_SCAN_FOR_MACRO_VIRUS)!=0)
	{
		if(strcmp(flag_macro_action,"Remove infected macros from docs")==0)
			add_to_command(" -saferemove");
		if(strcmp(flag_macro_action,"Remove all macros from infected docs")==0)
			add_to_command(" -removenew");
		if(strcmp(flag_macro_action,"Remove all macros from all docs")==0)
			add_to_command(" -removeall");
	}
	/*  -wrap switch 	is not supported, if you want to edit the logs use a text editor */
	/*  -nobreak    	is not supported (Do not abort scan if ESC is pressed) */
	/*  -page       	is not supported (Pause after each page.)*/
	/*  -silent     	is not supported (Do not generate any screen output.)*/
	add_to_command(" &");
	debug_printf("do_scan_f() %s \n",command);
	system(command);
	free(command);
	free(report_file);
}

void scan_f()
{
	debug_puts("scan_f()");

	prepare_report_path();

	if(lstat(report_file,&stat_p)==0) /* File exists */
	{
		if(!S_ISREG(stat_p.st_mode)) /* is not a regular file */
		{
			dialog_window("Alert", "Report file is not a regular file!",ONE_BUTTON, MODAL, NULL);
			debug_puts("Error: Report file is not a regular file! Aborting scan.");
			free(report_file);
			return;
		}
		else /* is a regular file */
		{
			if(strcmp(flag_append,"Append to existing report file")!=0) /* Don't append */
			{
				if(dialog_window("Alert", "Report file exists. Overwrite?", ONE_BUTTON, MODAL, NULL)==1) /* Don't overwrite */
				{
					debug_printf("Report file exists. Overwrite? dialog_retval %d\n",dialog_retval);
					free(report_file);
					return;
				}
				debug_printf("Report file exists. Overwrite? dialog_retval %d\n",dialog_retval);
				/*else overwrite existing report */
			}
			/*else append to existing report */
		}
	}
	/*else file don't exist so write it now */
	do_scan_f();
}

/*############################## End of Scan button function  ######################################################*/

void remove_lock()
{
	debug_puts("remove_lock()");

	if(remove(LOCK_FILE)==-1)
	{
		puts("Failed to remove lock file");
		dialog_window("Error","Failed to remove lock file.\nAborting...", ONE_BUTTON, MODAL, NULL);
		exit(EXIT_FAILURE);
	}
}

void about_f()
{
	int retval;

	debug_puts("about_f()");

	if(read_license==0)
	{
retry:
		retval=dialog_window(VERSION, VERSION"\n"COPYRIGHT"\n"LICENSE"\n"HOMEPAGE
									"\n"WARRANTY_LINE_0"\n"WARRANTY_LINE_1
									"\nDo you agree?",THREE_BUTTONS, MODAL,"License");
		if(retval==0)
			read_license=1;	/* 0 = Yes  */
		if(retval==1)			/* 1 = No   */
		{
			remove_lock();
			exit(EXIT_SUCCESS);
		}
		if(retval==2)			/* 2 = License */
		{
			debug_puts(XFPROT_BIN_DIR LICENSE_FILE);
			fd = fopen(XFPROT_BIN_DIR LICENSE_FILE, "r");
			textview_window_create("GPL License",620,430,8,MODAL,fd,NULL);
			if(fd)
				fclose(fd);
			goto retry;
		}
	}
	else
		dialog_window(VERSION, VERSION"\n"COPYRIGHT"\n"LICENSE"\n"HOMEPAGE
			"\n"WARRANTY_LINE_0"\n"WARRANTY_LINE_1,ONE_BUTTON, MODAL, NULL);
}

void root_warn()
{
	if(pw->pw_uid==0 )
	{
		debug_puts("root_warn()");

		puts("You are root");
		if(dialog_window("Warning", "Warning, you are running xfprot as root!\nContinue?",
								TWO_BUTTONS, MODAL, NULL)==1) /* 1 = No */
		{
			remove_lock();
			exit(EXIT_SUCCESS);
		}
	}
}

void do_lock_file()
{
	int pfd;
	char str[DEF_SIZE];

	debug_puts("do_lock_file()");

	/* Check if LOCK FILE exists and remove if stale */
	if(lstat(LOCK_FILE,&stat_p)==0) /* File exists */
	{
		puts("LOCK_FILE found: Another instance of xfprot is running?");

		dialog_window("Warning","Warning, xfprot has detected a lock file.\n"
							"Another instance of xfprot seems to be running.\n"
							"If you have determined that there is no other xfprot running\n"
							"please click \"Yes\" to begin.",TWO_BUTTONS, MODAL, NULL);
		if(dialog_retval==1)
			exit(EXIT_SUCCESS);
		else
			remove_lock();
	}

	if ((pfd=open(LOCK_FILE,O_RDWR|O_CREAT|O_EXCL,0640))<0)
	{
		puts("Cannot create lock file or lock file already exists");
		exit(EXIT_FAILURE);			/* cannot open */
	}
	if (lockf(pfd,F_TLOCK,0)<0)
	{
		puts("lockf(): failed");
		exit(EXIT_FAILURE);			/* cannot lock */
	}
	/* only first instance continues */

	sprintf(str,"%d\n",getpid());
	printf("Writing pid to lock file: %d\n",atoi(str));
	/* record pid to lockfile */
	write(pfd,str,strlen(str));
	close(pfd);
}

void set_dir_properties()
{
	debug_puts("set_dir_properties()");

	puts("Setting permissions of private dir to 0700");
	xchmod(XFPROT_PRIV_DIR,0700, EXIT_ON_FAILURE);
	puts("Setting owner of private dir");
	xchown(XFPROT_PRIV_DIR, pw->pw_uid, pw->pw_gid, EXIT_ON_FAILURE);
	puts("chdir() to private dir");
	xchdir(XFPROT_PRIV_DIR, EXIT_ON_FAILURE);
}


void check_for_bin()
{
	debug_puts("check_for_bin()");

	printf("Checking for %s ...",PATH_TO_F_PROT_BIN);

	/* access  may  not  work  correctly  on NFS file systems */
	if (access(PATH_TO_F_PROT_BIN , X_OK)<0)
	{
		puts(" not found");
		if(dialog_window("Error","Error, " PATH_TO_F_PROT_BIN " not found!\nAborting...",
								 ONE_BUTTON, MODAL, NULL)==0)
			exit(EXIT_FAILURE);
	}
	puts(" found");
}

void toggle1_f()
{
	debug_puts("toggle1_f()");

	if((strcmp(flag_scanmode,"Interactive")==0 ||
		strcmp(flag_scanmode,"Automatic")==0 ) &&
		strcmp(flag_macro,ONLY_SCAN_FOR_MACRO_VIRUS)!=0)
	{
		/* It's better to reset the flags to default values
			here to avoid leftovers from previous toggles */
		gtk_widget_show(action_table);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (disinfect_button),	TRUE);
		sprintf(flag_disinf,"%s","Disinfect");
		debug_printf("flag_disinf %s\n",flag_disinf);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rename_button),	FALSE);
		sprintf(flag_rename,"%s","");
		debug_printf("flag_rename %s\n",flag_rename);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (delete_button),	FALSE);
		sprintf(flag_delete,"%s","");
		debug_printf("flag_delete %s\n",flag_delete);
	}
	else
	{
		gtk_widget_hide(action_table);
	}
}

void toggle2_f()
{
	debug_puts("toggle2_f()");

	if((strcmp(flag_scanmode,"Interactive")==0 ||
		strcmp(flag_scanmode,"Automatic")==0)  &&
		strcmp(flag_macro,NOT_SCAN_FOR_MACRO_VIRUS)!=0)
	{
		/* It's better to reset the flags to default values
			here to avoid leftovers from previous toggles */
		gtk_widget_show(macro_remove_table);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (macro_1_button), TRUE);
		sprintf(flag_macro_action,"%s","Remove infected macros from docs");
		debug_printf("flag_macro_action %s\n",flag_macro_action);
	}
	else
	{
		gtk_widget_hide(macro_remove_table);
	}
}

/* This callback quits the program */
void delete_event( void )
{
	debug_puts("delete_event()");
	remove_lock();
	gtk_main_quit ();
	exit(EXIT_SUCCESS);
}

void flag_scanmode_callback(GtkWidget *widget, gpointer callback_data )
{
	sprintf(flag_scanmode,"%s",(char *)callback_data);
	debug_printf("flag_scanmode %s\n",flag_scanmode);
	toggle1_f();
	toggle2_f();
}

void flag_file_types_callback(GtkWidget *widget, gpointer callback_data )
{
	sprintf(flag_file_types,"%s",(char *)callback_data);
	debug_printf("flag_file_types %s\n",flag_file_types);
}

void flag_macro_callback(GtkWidget *widget, gpointer callback_data )
{
	sprintf(flag_macro,"%s",(char *)callback_data);
	debug_printf("flag_macro %s\n",flag_macro);
	toggle2_f();
	toggle1_f();
}

void flag_macro_action_callback(GtkWidget *widget, gpointer callback_data )
{
	sprintf(flag_macro_action,"%s",(char *)callback_data);
	debug_printf("flag_macro_action %s\n",flag_macro_action);
}

void flag_heur_callback(GtkWidget *widget, gpointer callback_data )
{
	sprintf(flag_heur,"%s",(char *)callback_data);
	debug_printf("flag_heur %s\n",flag_heur);
}

void flag_callback(GtkWidget *widget, gpointer callback_data )
{
	char * ptr=NULL;

	if(strcmp(callback_data,"Do not scan subdirectories")==0)
		ptr=flag_nosub;
	if(strcmp(callback_data,"Disinfect")==0)
		ptr=flag_disinf;
	if(strcmp(callback_data,"Rename")==0)
		ptr=flag_rename;
	if(strcmp(callback_data,"Delete")==0)
		ptr=flag_delete;
	if(strcmp(callback_data,"Append to existing report file")==0)
		ptr=flag_append;
	if(strcmp(callback_data,"List all scanned files")==0)
		ptr=flag_list;
	if(strcmp(callback_data,"Scan inside archives")==0)
		ptr=flag_archives;
	if(strcmp(callback_data,"Scan compressed executables")==0)
		ptr=flag_comp_exe;
	if(strcmp(callback_data,"Scan for other various malware")==0)
		ptr=flag_collect;
	if(strcmp(callback_data,"Enable neural network virus detection")==0)
		ptr=flag_ai;
	if(strcmp(callback_data,"Follow symbolic links")==0)
		ptr=flag_follow;
	if(strcmp(callback_data,"Do not complain about old DEF files")==0)
		ptr=flag_old;

	if(strlen(ptr)==0)
	{
		sprintf(ptr,"%s",(char *)callback_data);
		debug_printf("set flag %s\n",ptr);
	}
	else
	{
		sprintf(ptr,"%s","");
		debug_printf("unset flag %s\n",ptr);
	}
}

void signal_handler( int sig)
{
	switch(sig)
	{
		case SIGINT:
				puts("SIGINT signal catched");
				delete_event();
				break;
		case SIGTERM:
				puts("SIGTERM signal catched");
				delete_event();
				break;
	}
}

int main(int argc, char **argv)
{
	GtkWidget 	*main_window,
					*label,
					*top_table,
					*mode_table,
					*macro_table,
					*file_types_table,
					*report_table,
					*option_table,
					*frame,
					*table,
					*button,
					*radio_button,
					*check_button;
	GtkTooltips *action_tooltip;
	GSList *scan_mode_group;
	GSList *scan_macro_group;
	GSList *remove_macro_group;
	GSList *heuristics_group;


	signal(SIGHUP,SIG_IGN);  				/* ignore SIGHUP */
	signal(SIGTERM,signal_handler);		/* catch SIGTERM */
	signal(SIGINT,signal_handler);		/* catch SIGINT  */

	gtk_set_locale();
	gtk_init (&argc, &argv);

	#ifdef NO_CORE_DUMPS
	/* make the process not dumpable under Linux 2.4, seems to work also for 2.6.0-test3 */
	prctl(PR_SET_DUMPABLE,0);
	printf("Process is%sdumpable\n",(prctl(PR_GET_DUMPABLE)==0)?" not ":" ");
	#endif

	/* Force the output to be unbuffered when debugging.*/
	#ifdef DEBUG
	setbuf(stdout, (char *) 0);
	#endif

	/* Command line args */
	if( argc == 2)
	{
		if(strcmp(argv[1],"-v")==0)
		{
			puts(VERSION);
			exit(EXIT_SUCCESS);
		}
		fprintf(stderr,"Unknown command line switch: %s\n",argv[1]);
		exit(EXIT_FAILURE);
	}
	if(argc>2)
	{
		puts("Too much args on command line");
		exit(EXIT_FAILURE);
	}

	/* Try to determine the home dir of the user in
		passwd struct to avoid use of getenv("HOME").
		Create a private xfprot dir in user's home dir
		if it doesn't already exist.
		Set explicitly ownership and permissions of xfprot's dir.
		If something goes wrong we issue an error and exit.
		BTW: the user is forced to write the report always in
		xfprot's  private dir.
	*/

	/* Who am I ? */
	if((pw=getpwuid(getuid()))==NULL)
	{
		puts("getpwuid(getuid()): failed");
		exit(EXIT_FAILURE);
	}
	/* chdir to user's home dir */
	if((xchdir(pw->pw_dir, RETURN_ON_FAILURE))==EXIT_ERROR)
	{
		/* else try "/" */
		puts("Trying \"/\".");
		xchdir("/",EXIT_ON_FAILURE);
	}
	/* Check for xfprot's private dir */
	if(lstat(XFPROT_PRIV_DIR,&stat_p)==0)
	{
		puts("Private dir found");
		if(S_ISLNK(stat_p.st_mode))
		{
			/* We don't want to be redirected by a link */
			puts("Private dir is a link");
			exit(EXIT_FAILURE);
		}
		if (S_ISDIR(stat_p.st_mode))
		{
			/* OK let's go on */
			puts("Private dir is not a link");
		}
	}
	else
	{
		puts("Private dir not found.\nMaking private dir.");
		xmkdir(XFPROT_PRIV_DIR, 0700, EXIT_ON_FAILURE);
	}

	/* Explicitly set private dir's permissions and owner */
	set_dir_properties();
	/* check for f-prot*/
	check_for_bin();
	/* lock file stuff */
	do_lock_file();
	/* display a warning if we are root */
	root_warn();
	/* display license */
	about_f();

	main_window = window_create(640,  450, 0, VERSION" "SHORT_COPYRIGHT,
											delete_event, NULL, NOT_MODAL);
	table = table_create(20, 10, main_window, 2 );


	frame = frame_create_on_table("Path to scan", table, 0, 10, 0,3);
	top_table = table_create(2, 20, frame, 2 );
	path=xgetcwd(path);
	debug_puts(path);
	path_entry_box = text_box_create_on_table(path, NOT_EDITABLE, top_table, 0, 16, 0, 1);
	free(path);
	button = button_create_on_table("Select", do_file_selection, NULL, top_table, 17, 20, 0,1);
	check_button = default_check_create_on_table("Do not scan subdirectories",
																flag_callback, INACTIVE, top_table, 0, 6, 1, 2);

	frame = frame_create_on_table("Scan mode", table, 0, 5, 3, 7);
	mode_table = table_create(3, 1, frame, 2 );
	radio_button = default_radio_create_on_table(NULL,"Report only",flag_scanmode_callback,
													ACTIVE,mode_table, 0, 1, 0,1);
	scan_mode_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_button));
	radio_button = default_radio_create_on_table(radio_button,"Interactive",flag_scanmode_callback,
													INACTIVE,mode_table, 0, 1, 1,2);
	radio_button = default_radio_create_on_table(radio_button,	"Automatic",flag_scanmode_callback,
													INACTIVE,mode_table, 0, 1, 2,3);

	frame = frame_create_on_table("Actions to try on infected files", table, 0, 5, 7, 11);
	action_table = table_create(3, 1, frame, 2 );
	action_tooltip=gtk_tooltips_new();

	/* Warning: Gtk+ 1.2.10 docs for gtk_tooltips_set_delay say:
	 * delay: an integer value representing seconds !!!
	 * but in gtktooltips.c  we read #define DEFAULT_DELAY 500   (Default delay in ms)
	 * So this seems to works for me only using seconds*1000
	 */
	gtk_tooltips_set_delay (action_tooltip,2000); /* 2 seconds */

	disinfect_button =	check_create_on_table(	"Disinfect whenever possible",	flag_callback,
																"Disinfect",	ACTIVE,	action_table, 0, 1, 0, 1);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),disinfect_button,
				"It is possible to specify combinations of switches",NULL);
	rename_button =		check_create_on_table(	"Rename infected COM/EXE files to VOM/VXE",flag_callback,
																"Rename",INACTIVE, action_table, 0, 1, 1, 2);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),rename_button,
				"It is possible to specify combinations of switches",NULL);
	delete_button =		check_create_on_table(	"Delete  infected files",	flag_callback,
																"Delete",	INACTIVE, action_table, 0, 1, 2, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),delete_button,
				"It is possible to specify combinations of switches",NULL);
	gtk_tooltips_enable (action_tooltip);

	frame = frame_create_on_table("File types", table, 0, 5, 11, 15);
	file_types_table = table_create(3, 1, frame, 2 );
	radio_button = default_radio_create_on_table(NULL,"Standard file extensions",flag_file_types_callback,
													 ACTIVE,file_types_table, 0, 1, 0, 1);
	scan_mode_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_button));
	radio_button = default_radio_create_on_table(radio_button,	"Attempt to identify files",flag_file_types_callback,
													INACTIVE,file_types_table, 0, 1, 1, 2);
	radio_button = default_radio_create_on_table(radio_button,	"Scan all files",flag_file_types_callback,
													INACTIVE,file_types_table, 0, 1, 2, 3);

	frame = frame_create_on_table("Report file", table, 0, 5, 15, 19);
	report_table = table_create(3, 1, frame, 2 );
	report_path_entry_box = text_box_create_on_table( DEF_REPORT_FILE , EDITABLE, report_table, 0, 1, 0, 1);
	check_button = default_check_create_on_table("Append to existing report file",
																flag_callback, INACTIVE, report_table, 0, 1, 1, 2);
	check_button = default_check_create_on_table("List all scanned files",
																flag_callback, INACTIVE, report_table, 0, 1, 2, 3);

	frame = frame_create_on_table("Scan for macro virus", table, 5, 10, 3, 7);
	macro_table = table_create(3, 1, frame, 2 );
	radio_button = radio_create_on_table(NULL,"Yes",flag_macro_callback, SCAN_FOR_MACRO_VIRUS,
													 ACTIVE,	 	macro_table, 0, 1, 0, 1);
	scan_macro_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_button));
	radio_button = radio_create_on_table(radio_button,"No",flag_macro_callback,NOT_SCAN_FOR_MACRO_VIRUS,
													 INACTIVE,	macro_table, 0, 1, 1, 2);
	radio_button = radio_create_on_table(radio_button,"Only",flag_macro_callback,	ONLY_SCAN_FOR_MACRO_VIRUS,
													 INACTIVE,	macro_table, 0, 1, 2, 3);

	frame = frame_create_on_table("Macro virus removal options", table, 5, 10, 7, 11);
	macro_remove_table = table_create(3, 1, frame, 2 );
	macro_1_button = default_radio_create_on_table(NULL, "Remove infected macros from docs",
																	flag_macro_action_callback,  ACTIVE, macro_remove_table, 0, 1, 0, 1);
	remove_macro_group = gtk_radio_button_group (GTK_RADIO_BUTTON (macro_1_button));
	macro_2_button = default_radio_create_on_table(macro_1_button,"Remove all macros from infected docs",
																	flag_macro_action_callback,INACTIVE, macro_remove_table, 0, 1, 1, 2);
	macro_3_button = default_radio_create_on_table(macro_2_button,"Remove all macros from all docs",
																	flag_macro_action_callback,INACTIVE, macro_remove_table, 0, 1, 2, 3);

	frame = frame_create_on_table("Options", table, 5, 10, 11, 19);
	option_table = table_create(7, 5, frame, 2 );
	check_button = 			check_create_on_table("Scan inside .ZIP and .ARJ files",	flag_callback,
													"Scan inside archives", INACTIVE, option_table, 0, 5, 0, 1);
	check_button = default_check_create_on_table("Scan compressed executables",
																flag_callback, INACTIVE, option_table, 0, 5, 1, 2);
	check_button = default_check_create_on_table("Scan for other various malware",
																flag_callback, INACTIVE, option_table, 0, 5, 2, 3);
	check_button = default_check_create_on_table("Enable neural network virus detection",
																flag_callback, INACTIVE, option_table, 0, 5, 3, 4);
	check_button = default_check_create_on_table("Follow symbolic links",
																flag_callback, INACTIVE, option_table, 0, 5, 4, 5);
	check_button = default_check_create_on_table("Do not complain about old DEF files",
																flag_callback, INACTIVE, option_table, 0, 5, 5, 6);

	label = label_create_on_table("Use heuristics:           ", option_table, 0, 2, 6,7);
	radio_button = radio_create_on_table(NULL				, "Yes",	 flag_heur_callback, USE_HEURISTICS,
																					 ACTIVE, 	option_table, 2, 3, 6, 7);
	heuristics_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radio_button));
	radio_button = radio_create_on_table(radio_button	, "No",	 flag_heur_callback, NOT_USE_HEURISTICS,
																					 INACTIVE, 	option_table, 3, 4, 6, 7);
	radio_button = radio_create_on_table(radio_button	, "Only", flag_heur_callback, ONLY_USE_HEURISTICS,
																					 INACTIVE, 	option_table, 4, 5, 6, 7);

	button = button_create_on_table("Exit"			, delete_event	, NULL, table, 9, 10, 19, 20);
	button = button_create_on_table("Info"			, fprot_info	, NULL, table, 7, 8,  19, 20);
	button = button_create_on_table("Virus list"	, fprot_virlist, NULL, table, 6, 7,  19, 20);
	button = button_create_on_table("Test"			, fprot_test	, NULL, table, 5, 6,  19, 20);
	button = button_create_on_table("Update"		, fprot_update	, NULL, table, 4, 5,  19, 20);
	button = button_create_on_table("Report"		, view_report	, NULL, table, 3, 4,  19, 20);
	button = button_create_on_table("Scan"			, scan_f			, NULL, table, 2, 3,  19, 20);
	button = button_create_on_table("About"		, about_f		, NULL, table, 0, 1,  19, 20);

	gtk_widget_show_all(main_window);
	toggle1_f();
	toggle2_f();
	gtk_main();

	return(EXIT_SUCCESS);
}
